/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Launch a Manager Event
 * 
 * Copyright (C) 2005, Anthony Minessale
 *
 * Anthony Minessale II <anthm@cylynx.com>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <asterisk/file.h>
#include <asterisk/logger.h>
#include <asterisk/channel.h>
#include <asterisk/pbx.h>
#include <asterisk/module.h>
#include <asterisk/manager.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>

static char *tdesc = "Event Application";
static char *app = "Event";
static char *synopsis = "Sends an Event from the dialplan or AGI\n\n"
"Usage: Event(Event=MyEvent|Name=fred|Info=${EXTEN})\n"
"Headers and thier values are up to you.\n"
"The Flag: header actually can control which channel the event comes in on\n"
"One of (system,call,log,verbose,command,agent,user)\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int event_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char event_data[1024];
	char tmp[256];
	char event[256];
	char *stack;
	char *var, *val;
	int ok=0 , flag = EVENT_FLAG_USER;

	memset(event_data, 0, sizeof(event_data));

	if (!data) {
		ast_log(LOG_WARNING, "event requires an argument\n");
		return -1;
	}
	LOCAL_USER_ADD(u);
	
	stack = (char *) data;
	val=stack;
	while(val) {
		val = strsep(&stack, "|");
		if (!val)
			break;
		var = strsep(&val, "=");
		if (!var)
			break;
		
		if (!strcasecmp(var, "event") && val) {
			strncpy(event, val, sizeof(event));
			ok++;
		}
		else {
			if (!strcasecmp(var, "flag") && val) {
				if (!strcasecmp(val, "system")) {
					flag = EVENT_FLAG_SYSTEM;
				} else if (!strcasecmp(val, "call")) {
					flag = EVENT_FLAG_CALL;
				} else if (!strcasecmp(val, "log")) {
					flag = EVENT_FLAG_LOG;
				} else if (!strcasecmp(val, "verbose")) {
					flag = EVENT_FLAG_VERBOSE;
				} else if (!strcasecmp(val, "command")) {
					flag = EVENT_FLAG_COMMAND;
				} else if (!strcasecmp(val, "agent")) {
					flag = EVENT_FLAG_AGENT;
				} else {
					flag = EVENT_FLAG_USER;
				}
			}

			snprintf(tmp, sizeof(tmp), "%s: %s\r\n", var, val);
			strncat(event_data, tmp, sizeof(event_data));
		}
	}

	if (!ok) {
		ast_log(LOG_WARNING, "Event requires at least an Event= arguement\n");
		return -1;
	}

	manager_event(flag, event, "%sExtension: %s\r\nChannel: %s\r\nUniqueid: %s\r\n", event_data, chan->exten, chan->name, chan->uniqueid);
		      

	LOCAL_USER_REMOVE(u);
	return res;
}



int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, event_exec, tdesc, synopsis);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
